@@ -340,7 +340,7 @@ variable: an unreadable object representing the error is printed instead.")
       o)))
 
 ;;; guts of PRINT-UNREADABLE-OBJECT
-(defun %print-unreadable-object (object stream type identity body)
+(defun %print-unreadable-object (object stream type identity &optional body)
   (declare (type (or null function) body))
   (if *print-readably*
       (print-not-readable-error object stream)
@@ -585,6 +585,7 @@ variable: an unreadable object representing the error is printed instead.")
 ;;; Output PNAME (a symbol-name or package-name) surrounded with |'s,
 ;;; and with any embedded |'s or \'s escaped.
 (defun output-quoted-symbol-name (pname stream)
+  (declare (string pname))
   (write-char #\| stream)
   (dotimes (index (length pname))
     (let ((char (schar pname index)))
@@ -1785,8 +1786,15 @@ variable: an unreadable object representing the error is printed instead.")
 
 (defun output-fdefn (fdefn stream)
   (print-unreadable-object (fdefn stream)
-    (write-string "FDEFINITION object for " stream)
-    (output-object (fdefn-name fdefn) stream)))
+    (write-string "FDEFINITION for " stream)
+    ;; It's somewhat unhelpful to print as <FDEFINITION for (SETF #)>
+    ;; Generalized function names are indivisible.
+    (let ((name (fdefn-name fdefn)))
+      (if (atom name)
+          (output-object name stream)
+          ;; This needn't protect against improper lists.
+          ;; (You'd get crashes in INTERNAL-NAME-P and other places)
+          (format stream "(~{~S~^ ~})" name)))))
 
 #!+sb-simd-pack
 (defun output-simd-pack (pack stream)
