@@ -49,3 +49,11 @@
                (cold-name (concatenate 'simple-string cold-name-prefix stem)))
           (rename-package package name (cons cold-name nicknames)))
         #+sb-package-locks (sb-ext:unlock-package package)))))
+
+;; This macro is never defined for the target Lisp,
+;; only the cross-compilation host (see "src/code/specializable-array")
+;; but it is needed to read x86-64/insts.lisp and other things.
+(export 'sb-int::!coerce-to-specialized 'sb-int)
+(defmacro sb-int:!coerce-to-specialized (a type)
+  (declare (ignore type))
+  a)
