@@ -9,7 +9,7 @@
  * files for more information.
  */
 
-#ifdef __FreeBSD__
+#if defined(LISP_FEATURE_FREEBSD)
 #include <osreldate.h>
 #endif
 
@@ -32,15 +32,7 @@ typedef vm_size_t os_vm_size_t;
 typedef off_t os_vm_offset_t;
 typedef int os_vm_prot_t;
 
-#if defined __OpenBSD__
-/* name defined for compatibility between OpenBSD 3.1 sigaltstack(2) and
- * Linux sigaltstack(2) */
-typedef struct sigaltstack stack_t;
-#elif defined __FreeBSD__
-/* FreeBSD 4.6 and NetBSD 1.6 already have stack_t defined. */
-#endif
-
-#if defined __FreeBSD__
+#if defined(LISP_FEATURE_FREEBSD)
 /* Note: The man page for sigaction(2) in FreeBSD 4.0 says that this
  * is an mcontext_t, but according to comments by Raymond Wiker in the
  * original FreeBSD port of SBCL, that's wrong, it's actually a
@@ -53,7 +45,6 @@ typedef ucontext_t os_context_t;
  * step flag bit by messing with the flags stored in a signal context,
  * so we need to implement single stepping in a more roundabout way. */
 #define CANNOT_GET_TO_SINGLE_STEP_FLAG
-#define SIG_MEMORY_FAULT SIGSEGV
 /* Sometime in late 2005 FreeBSD was changed to signal SIGSEGV instead
  * of SIGBUS for memory faults, as required by POSIX. In order to
  * support both new and old FreeBSD at the same time, both signals are
@@ -62,16 +53,29 @@ typedef ucontext_t os_context_t;
  * quaint memories, feel free to delete this hack (and any code that's
  * #ifdef SIG_MEMORY_FAULT2'ed). -- JES, 2005-12-30
  */
-#define SIG_MEMORY_FAULT2 SIGBUS
+/* Hooking both SIGBUS and SIGSEGV causes troubles on some situation,
+ * so use a variable.
+ */
+extern int sig_memory_fault;
+#define SIG_MEMORY_FAULT (sig_memory_fault)
 
-#define SIG_INTERRUPT_THREAD (SIGINFO)
-#define SIG_STOP_FOR_GC (SIGUSR1)
-#define SIG_RESUME_FROM_GC (SIGUSR2)
+#define SIG_STOP_FOR_GC (SIGUSR2)
+
+#elif defined __DragonFly__
+
+#include <sys/ucontext.h>
+typedef ucontext_t os_context_t;
+
+#define SIG_MEMORY_FAULT (SIGSEGV)
+#define SIG_STOP_FOR_GC (SIGUSR2)
 
 #elif defined __OpenBSD__
 
 typedef struct sigcontext os_context_t;
 #define SIG_MEMORY_FAULT SIGSEGV
+#if defined(LISP_FEATURE_X86)
+extern int openbsd_use_fxsave;
+#endif
 
 #elif defined __NetBSD__
 
@@ -79,6 +83,8 @@ typedef struct sigcontext os_context_t;
 typedef ucontext_t os_context_t;
 #define SIG_MEMORY_FAULT SIGSEGV
 
+#define SIG_STOP_FOR_GC (SIGUSR2)
+
 #elif defined LISP_FEATURE_DARWIN
 #include "darwin-os.h"
 #else
