@@ -33,23 +33,23 @@
   (slot-value x 'c))
 
 (let ((fun (compile nil '(lambda (x) (slot-value x 'a)))))
-  (dotimes (i 4) ; KLUDGE: get caches warm
+  (dotimes (i 4)                        ; KLUDGE: get caches warm
     (assert (= 1 (slot-value *foo* 'a)))
     (assert (= 1 (a-of *foo*)))
     (assert (= 1 (funcall fun *foo*)))
-    (assert (raises-error? (b-of *foo*)))
-    (assert (raises-error? (c-of *foo*)))))
+    (assert-error (b-of *foo*))
+    (assert-error (c-of *foo*))))
 
 (defclass foo ()
   ((b :initarg :b :initform 3) (a :initarg :a)))
 
 (let ((fun (compile nil '(lambda (x) (slot-value x 'a)))))
-  (dotimes (i 4) ; KLUDGE: get caches warm
+  (dotimes (i 4)                        ; KLUDGE: get caches warm
     (assert (= 1 (slot-value *foo* 'a)))
     (assert (= 1 (a-of *foo*)))
     (assert (= 1 (funcall fun *foo*)))
     (assert (= 3 (b-of *foo*)))
-    (assert (raises-error? (c-of *foo*)))))
+    (assert-error (c-of *foo*))))
 
 (defclass foo ()
   ((c :initarg :c :initform t :allocation :class)
@@ -81,12 +81,12 @@
   ((b :initarg :b :initform 3)))
 
 (let ((fun (compile nil '(lambda (x) (slot-value x 'a)))))
-  (dotimes (i 4) ; KLUDGE: get caches warm
-    (assert (raises-error? (slot-value *foo* 'a)))
-    (assert (raises-error? (a-of *foo*)))
-    (assert (raises-error? (funcall fun *foo*)))
+  (dotimes (i 4)                        ; KLUDGE: get caches warm
+    (assert-error (slot-value *foo* 'a))
+    (assert-error (a-of *foo*))
+    (assert-error (funcall fun *foo*))
     (assert (= 3 (b-of *foo*)))
-    (assert (raises-error? (c-of *foo*)))))
+    (assert-error (c-of *foo*))))
 
 ;;; test that :documentation argument to slot specifiers are used as
 ;;; the docstrings of accessor methods.
@@ -123,7 +123,7 @@
 
 ;;; Tests the compiler's incremental rejiggering of GF types.
 (fmakunbound 'foo)
-(with-test (:name keywords-supplied-in-methods-ok-1)
+(with-test (:name :keywords-supplied-in-methods-ok-1)
   (assert
    (null
     (nth-value
@@ -134,7 +134,7 @@
        (compile nil '(lambda () (foo (read) :bar 10))))))))
 
 (fmakunbound 'foo)
-(with-test (:name keywords-supplied-in-methods-ok-2)
+(with-test (:name :keywords-supplied-in-methods-ok-2)
   (assert
    (nth-value
     1
@@ -149,7 +149,7 @@
 ;; parse the tail of the arglist as keywords, so we don't treat the
 ;; function type as having &KEY in it.
 (fmakunbound 'foo)
-(with-test (:name gf-rest-method-key)
+(with-test (:name :gf-rest-method-key)
   (defgeneric foo (x &rest y))
   (defmethod foo ((i integer) &key w) (list i w))
   ;; 1.0.20.30 failed here.
@@ -161,7 +161,7 @@
 ;; If the GF has &KEY and &ALLOW-OTHER-KEYS, the methods' keys can be
 ;; anything, and we don't warn about unrecognized keys.
 (fmakunbound 'foo)
-(with-test (:name gf-allow-other-keys)
+(with-test (:name :gf-allow-other-keys)
   (defgeneric foo (x &key &allow-other-keys))
   (defmethod foo ((i integer) &key y z) (list i y z))
   (assert
@@ -176,7 +176,7 @@
 ;; If any method has &ALLOW-OTHER-KEYS, 7.6.4 point 5 seems to say the
 ;; GF should be construed to have &ALLOW-OTHER-KEYS.
 (fmakunbound 'foo)
-(with-test (:name method-allow-other-keys)
+(with-test (:name :method-allow-other-keys)
   (defgeneric foo (x &key))
   (defmethod foo ((x integer) &rest y &key &allow-other-keys) (list x y))
   (assert (null (nth-value 1 (compile nil '(lambda () (foo 10 :foo 20))))))
